/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-audio.h"

#define OGMRIP_AUDIO_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_AUDIO, OGMRipAudioPriv))

struct _OGMRipAudioPriv
{
  guint aid;
  guint quality;

  gboolean normalize;

  OGMDvdAudioStream *stream;
};

static void ogmrip_audio_finalize (GObject *gobject);

G_DEFINE_ABSTRACT_TYPE (OGMRipAudio, ogmrip_audio, OGMRIP_TYPE_CODEC)

static void
ogmrip_audio_class_init (OGMRipAudioClass *klass)
{
  GObjectClass *gobject_class;

  gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->finalize = ogmrip_audio_finalize;

  g_type_class_add_private (klass, sizeof (OGMRipAudioPriv));
}

static void
ogmrip_audio_init (OGMRipAudio *audio)
{
  audio->priv = OGMRIP_AUDIO_GET_PRIVATE (audio);
  audio->priv->quality = 3;
}

static void
ogmrip_audio_finalize (GObject *gobject)
{
  OGMRipAudio *audio;

  audio = OGMRIP_AUDIO (gobject);
  if (audio->priv->stream)
    ogmdvd_audio_stream_unref (audio->priv->stream);

  G_OBJECT_CLASS (ogmrip_audio_parent_class)->finalize (gobject);
}

void
ogmrip_audio_set_quality (OGMRipAudio *audio, guint quality)
{
  g_return_if_fail (OGMRIP_IS_AUDIO (audio));

  audio->priv->quality = CLAMP (quality, 0, 10);
}

gint
ogmrip_audio_get_quality (OGMRipAudio *audio)
{
  g_return_val_if_fail (OGMRIP_IS_AUDIO (audio), -1);

  return audio->priv->quality;
}

void
ogmrip_audio_set_normalize (OGMRipAudio *audio, gboolean normalize)
{
  g_return_if_fail (OGMRIP_IS_AUDIO (audio));

  audio->priv->normalize = normalize;
}

gboolean
ogmrip_audio_get_normalize (OGMRipAudio *audio)
{
  g_return_val_if_fail (OGMRIP_IS_AUDIO (audio), FALSE);

  return audio->priv->normalize;
}

void
ogmrip_audio_set_dvd_audio_stream (OGMRipAudio *audio, OGMDvdAudioStream *stream)
{
  g_return_if_fail (OGMRIP_IS_AUDIO (audio));
  g_return_if_fail (stream != NULL);

  ogmdvd_audio_stream_ref (stream);

  if (audio->priv->stream)
    ogmdvd_audio_stream_unref (audio->priv->stream);

  audio->priv->stream = stream;

  ogmrip_codec_set_input (OGMRIP_CODEC (audio), 
      ogmdvd_audio_stream_get_video_stream (stream));
}

OGMDvdAudioStream *
ogmrip_audio_get_dvd_audio_stream (OGMRipAudio *audio)
{
  g_return_val_if_fail (OGMRIP_IS_AUDIO (audio), NULL);

  return audio->priv->stream;
}

