/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"

#include "ogmrip-backend.h"
#include "ogmrip-tools.h"
#include "ogmrip-lavc.h"
#include "ogmrip-xvid.h"
#include "ogmrip-avi.h"
#include "ogmrip-ogg.h"

#ifdef HAVE_MKV_SUPPORT
#include "ogmrip-mkv.h"
#endif

#include <stdio.h>
#include <string.h>
#include <glib/gstdio.h>

#define OGMRIP_XVID_DEFAULT_OPTIONS "chroma_opt:chroma_me:hq_ac:autoaspect:vhq=4"
#define OGMRIP_LAVC_DEFAULT_OPTIONS "vcodec=mpeg4:mbd=2:mv0:qprd:autoaspect:keyint=125"
#define OGMRIP_X264_DEFAULT_OPTIONS "bframes=2:b_pyramid"

gdouble
ogmrip_backend_wav_watch (OGMSpExec *exec, const gchar *buffer, OGMRipAudio *audio)
{
  gchar a[10], v[10], av[15], ct[15];
  gint frame, decoded;

  if (sscanf (buffer, "A:%s V:%s A-V:%s ct:%s %d/%d", a, v, av, ct, &frame, &decoded) == 6)
    return decoded / (gdouble) ogmrip_codec_get_length (OGMRIP_CODEC (audio), NULL);

  return -1.0;
}

gdouble
ogmrip_backend_acopy_watch (OGMSpExec *exec, const gchar *buffer, OGMRipAudio *audio)
{
  gint frames, progress;
  gchar pos[10];

  if (sscanf (buffer, "Pos:%s %df (%d%%)", pos, &frames, &progress) == 3)
    return frames / (gdouble) ogmrip_codec_get_length (OGMRIP_CODEC (audio), NULL);

  return -1.0;
}

gdouble
ogmrip_backend_xvid_watch (OGMSpExec *exec, const gchar *buffer, OGMRipVideo *video)
{
  gint frames, progress;
  gchar pos[10];

  if (sscanf (buffer, "Pos:%s %df (%d%%)", pos, &frames, &progress) == 3)
    return frames / (gdouble) ogmrip_codec_get_length (OGMRIP_CODEC (video), NULL);

  return -1.0;
}

gdouble
ogmrip_backend_lavc_watch (OGMSpExec *exec, const gchar *buffer, OGMRipVideo *video)
{
  gint frames, progress;
  gchar pos[10];

  if (sscanf (buffer, "Pos:%s %df (%d%%)", pos, &frames, &progress) == 3)
    return frames / (gdouble) ogmrip_codec_get_length (OGMRIP_CODEC (video), NULL);

  return -1.0;
}

#ifdef HAVE_X264_SUPPORT
gdouble
ogmrip_backend_x264_watch (OGMSpExec *exec, const gchar *buffer, OGMRipVideo *video)
{
  gint frames, progress;
  gchar pos[10];

  if (sscanf (buffer, "Pos:%s %df (%d%%)", pos, &frames, &progress) == 3)
    return frames / (gdouble) ogmrip_codec_get_length (OGMRIP_CODEC (video), NULL);

  return -1.0;
}
#endif /* HAVE_X264_SUPPORT */

#ifdef OGMRIP_THEORA_SUPPORT
gdouble
ogmrip_backend_yuv4mpeg_watch (OGMSpExec *exec, const gchar *buffer, OGMRipVideo *video)
{
  gchar v[10];
  gint frame, decoded;

  if (sscanf (buffer, "V:%s %d/%d", v, &frame, &decoded) == 3)
    return decoded / (gdouble) ogmrip_codec_get_length (OGMRIP_CODEC (video), NULL);

  return -1.0;
}
#endif /* OGMRIP_THEORA_SUPPORT */

gdouble
ogmrip_backend_crop_watch (OGMSpExec *exec, const gchar *buffer, OGMRipVideo *video)
{
  static guint frame = 0;
  static gint crop_x = 0, crop_y = 0, 
              crop_width = G_MAXINT, crop_height = G_MAXINT;

  if (g_str_has_prefix (buffer, "crop area:"))
  {
    gchar *str;

    str = strstr (buffer, "-vf crop=");
    if (str)
    {
      gint cx, cy, cw, ch;

      if (sscanf (str, "-vf crop=%d:%d:%d:%d", &cw, &ch, &cx, &cy) == 4)
      {
        crop_x = MAX (cx, crop_x);
        crop_y = MAX (cy, crop_y);
        crop_width = MAX (0, MIN (cw, crop_width));
        crop_height = MAX (0, MIN (ch, crop_height));

        ogmrip_video_set_crop_size (video, crop_x, crop_y, crop_width, crop_height);
      }

      frame ++;
      if (frame > 25)
        return -1.0;

      return frame / 25.;
    }
  }

  return -1.0;
}

gdouble
ogmrip_backend_ogg_merge_watch (OGMSpExec *exec, const gchar *buffer, OGMRipContainer *ogg)
{
  gulong frames, total;
  guint percent;
  gint steps;

  steps = ogmrip_container_get_tnumber (ogg) > 1 ? 2 : 1;

  if (sscanf (buffer, "progress: %ld/%ld frames (%d%%)", &frames, &total, &percent) == 3)
    return percent / (steps * 100.0);

  return -1.0;
}

gdouble
ogmrip_backend_ogg_split_watch (OGMSpExec *exec, const gchar *buffer, OGMRipContainer *ogg)
{
  gulong frames, total;
  guint percent;
/*
  if (sscanf (buffer, "progress: %ld/%ld frames (%d%%)", &frames, &total, &percent) == 3)
    return 0.5 + percent / 600.0;
*/
  if (sscanf (buffer, "Processing bytes %ld/%ld (%d%%)", &frames, &total, &percent) == 3)
    return 0.5 + percent / 400.0;
  else if (sscanf (buffer, "Processing frame %ld/%ld (%d%%)", &frames, &total, &percent) == 3)
    return 0.5 + percent / 400.0;

  return -1.0;
}

#ifdef HAVE_MKV_SUPPORT
gdouble
ogmrip_backend_matroska_watch (OGMSpExec *exec, const gchar *buffer, OGMRipContainer *matroska)
{
  gulong frames, total;
  guint percent;

  if (sscanf (buffer, "progress: %ld/%ld frames (%u%%)", &frames, &total, &percent) == 3)
    return percent / 100.0;
  else if (sscanf (buffer, "progress: %u%%", &percent) == 1)
    return percent / 100.0;

  return -1.0;
}
#endif /* HAVE_MKV_SUPPORT */

gdouble
ogmrip_backend_avi_watch (OGMSpExec *exec, const gchar *buffer, OGMRipContainer *avi)
{
  gint frames, progress;
  gchar pos[10];

  if (sscanf (buffer, "Pos:%s %df (%d%%)", pos, &frames, &progress) == 3)
    return progress / 100.;

  return -1.0;
}

gdouble
ogmrip_backend_vobsub_watch (OGMSpExec *exec, const gchar *buffer, OGMRipSubp *subp)
{
  gint frames, progress;
  gchar pos[10];

  if (sscanf (buffer, "Pos:%s %df (%d%%)", pos, &frames, &progress) == 3)
    return 0.98 * frames / (gdouble) ogmrip_codec_get_length (OGMRIP_CODEC (subp), NULL);

  return -1.0;
}

gdouble
ogmrip_backend_pgm_watch (OGMSpExec *exec, const gchar *buffer, OGMRipSubp *subp)
{
  guint files;

  if (sscanf (buffer, "%u files generated", &files) == 1)
  {
    g_object_set_data (G_OBJECT (subp), "__ogmrip_srt_index__", GUINT_TO_POINTER (0));
    g_object_set_data (G_OBJECT (subp), "__ogmrip_srt_files__", GUINT_TO_POINTER (files));
  }

  return -1.0;
}

gdouble
ogmrip_backend_ocr_watch (OGMSpExec *exec, const gchar *buffer, OGMRipSubp *subp)
{
  if (strncmp (buffer, "Elapsed time:", 13) == 0)
  {
    guint index, files;

    index = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (subp), "__ogmrip_srt_index__"));
    files = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (subp), "__ogmrip_srt_files__"));

    g_object_set_data (G_OBJECT (subp), "__ogmrip_srt_index__", GUINT_TO_POINTER (index + 1));

    return 0.98 + 0.02 * (index + 1) / (gdouble) files;
  }

  return -1.0;
}

gdouble
ogmrip_backend_dvdcpy_watch (OGMSpExec *exec, const gchar *buffer, OGMRipVideo *video)
{
  long long bytes, total;
  guint percent;

  if (sscanf (buffer, "%llu/%llu bytes written (%u%%)", &bytes, &total, &percent) == 3)
    return percent / 100.;

  return -1;
}

gchar **
ogmrip_backend_wav_command (OGMRipAudio *audio, gboolean header, const gchar *input, const gchar *output)
{
  OGMDvdVideoStream *vstream;
  OGMDvdAudioStream *astream;

  GPtrArray *argv;
  const gchar *device;
  gint vid, aid, start, end;

  g_return_val_if_fail (OGMRIP_IS_AUDIO (audio), NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (audio));
  g_return_val_if_fail (output != NULL, NULL);

  vstream = ogmrip_codec_get_input (OGMRIP_CODEC (audio));
  g_return_val_if_fail (vstream != NULL, NULL);

  astream = ogmrip_audio_get_dvd_audio_stream (audio);
  g_return_val_if_fail (astream != NULL, NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("mplayer"));
  g_ptr_array_add (argv, g_strdup ("-nocache"));
  g_ptr_array_add (argv, g_strdup ("-noframedrop"));

  g_ptr_array_add (argv, g_strdup ("-vc"));
  g_ptr_array_add (argv, g_strdup ("null"));
  g_ptr_array_add (argv, g_strdup ("-vo"));
  g_ptr_array_add (argv, g_strdup ("null"));
  g_ptr_array_add (argv, g_strdup ("-ao"));

#if MPLAYER_PRE >= 7
  if (header)
    g_ptr_array_add (argv, g_strdup_printf ("pcm:waveheader:file=%s", output));
  else
    g_ptr_array_add (argv, g_strdup_printf ("pcm:nowaveheader:file=%s", output));
#else /* MPLAYER_PRE */
  g_ptr_array_add (argv, g_strdup ("pcm"));
  if (header)
    g_ptr_array_add (argv, g_strdup ("-waveheader"));
  else
    g_ptr_array_add (argv, g_strdup ("-nowaveheader"));
  g_ptr_array_add (argv, g_strdup ("-aofile"));
  g_ptr_array_add (argv, g_strdup (output));
#endif /* MPLAYER_PRE */

  if (ogmrip_audio_get_normalize (audio))
  {
#if MPLAYER_PRE >= 6
    g_ptr_array_add (argv, g_strdup ("-af"));
    g_ptr_array_add (argv, g_strdup ("volnorm"));
#else /* MPLAYER_PRE */
    g_ptr_array_add (argv, g_strdup ("-aop"));
    g_ptr_array_add (argv, g_strdup ("list=volnorm"));
#endif /* MPLAYER_PRE */
  }

  ogmrip_codec_get_chapters (OGMRIP_CODEC (audio), &start, &end);
  g_ptr_array_add (argv, g_strdup ("-chapter"));
  if (end > -1)
    g_ptr_array_add (argv, g_strdup_printf ("%d-%d", start + 1, end + 1));
  else
    g_ptr_array_add (argv, g_strdup_printf ("%d", start + 1));

  aid = ogmdvd_audio_stream_get_id (astream);
  g_ptr_array_add (argv, g_strdup ("-aid"));
  switch (ogmdvd_audio_stream_get_format (astream))
  {
    case OGMDVD_AUDIO_FORMAT_LPCM:
      g_ptr_array_add (argv, g_strdup_printf ("%d", aid + 160));
      break;
    default:
      g_ptr_array_add (argv, g_strdup_printf ("%d", aid + 128));
      break;
  }

  device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (vstream));
  g_ptr_array_add (argv, g_strdup ("-dvd-device"));
  g_ptr_array_add (argv, g_strdup (device));

  vid = ogmdvd_video_stream_get_id (vstream);

#if MPLAYER_MAJOR > 0
  g_ptr_array_add (argv, g_strdup_printf ("dvd://%d", vid + 1));
#else /* MPLAYER_MAJOR */
  g_ptr_array_add (argv, g_strdup ("-dvd"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));
#endif /* MPLAYER_MAJOR */

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

gchar **
ogmrip_backend_acopy_command (OGMRipAudio *audio, const gchar *input, const gchar *output)
{
  OGMDvdVideoStream *vstream;
  OGMDvdAudioStream *astream;

  GPtrArray *argv;
  const gchar *device;
  gint vid, aid, start, end;

  g_return_val_if_fail (OGMRIP_IS_AUDIO (audio), NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (audio));
  g_return_val_if_fail (output != NULL, NULL);

  vstream = ogmrip_codec_get_input (OGMRIP_CODEC (audio));
  g_return_val_if_fail (vstream != NULL, NULL);

  astream = ogmrip_audio_get_dvd_audio_stream (audio);
  g_return_val_if_fail (astream != NULL, NULL);

  argv = g_ptr_array_new ();

  g_ptr_array_add (argv, g_strdup ("mencoder"));
  g_ptr_array_add (argv, g_strdup ("-nocache"));

  g_ptr_array_add (argv, g_strdup ("-ovc"));
  g_ptr_array_add (argv, g_strdup ("frameno"));
  g_ptr_array_add (argv, g_strdup ("-oac"));
  g_ptr_array_add (argv, g_strdup ("copy"));

  ogmrip_codec_get_chapters (OGMRIP_CODEC (audio), &start, &end);
  g_ptr_array_add (argv, g_strdup ("-chapter"));
  if (end > -1)
    g_ptr_array_add (argv, g_strdup_printf ("%d-%d", start + 1, end + 1));
  else
    g_ptr_array_add (argv, g_strdup_printf ("%d", start + 1));

  aid = ogmdvd_audio_stream_get_id (astream);
  g_ptr_array_add (argv, g_strdup ("-aid"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", aid + 128));

  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup (output));

  device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (vstream));
  g_ptr_array_add (argv, g_strdup ("-dvd-device"));
  g_ptr_array_add (argv, g_strdup (device));

  vid = ogmdvd_video_stream_get_id (vstream);

#if MPLAYER_MAJOR > 0
  g_ptr_array_add (argv, g_strdup_printf ("dvd://%d", vid + 1));
#else /* MPLAYER_MAJOR */
  g_ptr_array_add (argv, g_strdup ("-dvd"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));
#endif /* MPLAYER_MAJOR */

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

gchar **
ogmrip_backend_vorbis_command (OGMRipAudio *audio, const gchar *input, const gchar *output)
{
  GPtrArray *argv;
  gint quality;

  g_return_val_if_fail (OGMRIP_IS_AUDIO (audio), NULL);
  g_return_val_if_fail (input != NULL, NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (audio));
  g_return_val_if_fail (output != NULL, NULL);

  quality = ogmrip_audio_get_quality (audio);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("oggenc"));
  g_ptr_array_add (argv, g_strdup ("-r"));
  g_ptr_array_add (argv, g_strdup ("-R"));
  g_ptr_array_add (argv, g_strdup ("48000"));
  g_ptr_array_add (argv, g_strdup ("-q"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", quality));
  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup (output));
  g_ptr_array_add (argv, g_strdup (input));
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

gchar **
ogmrip_backend_mp3_command (OGMRipAudio *audio, const gchar *input, const gchar *output)
{
  static gchar *presets[] =
  {
    "64",
    "80",
    "96",
    "112",
    "128",
    "medium",
    "standard",
    "extreme",
    "256",
    "insane",
    "512"
  };

  GPtrArray *argv;
  gint quality;

  g_return_val_if_fail (OGMRIP_IS_AUDIO (audio), NULL);
  g_return_val_if_fail (input != NULL, NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (audio));
  g_return_val_if_fail (output != NULL, NULL);

  quality = ogmrip_audio_get_quality (audio);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("lame"));
  g_ptr_array_add (argv, g_strdup ("--nohist"));
  g_ptr_array_add (argv, g_strdup ("-h"));
  g_ptr_array_add (argv, g_strdup ("-t"));
  g_ptr_array_add (argv, g_strdup ("-r"));
  g_ptr_array_add (argv, g_strdup ("-x"));
  g_ptr_array_add (argv, g_strdup ("-s"));
  g_ptr_array_add (argv, g_strdup ("48"));
  g_ptr_array_add (argv, g_strdup ("--preset"));
  g_ptr_array_add (argv, g_strdup (presets[quality]));
  g_ptr_array_add (argv, g_strdup (input));
  g_ptr_array_add (argv, g_strdup (output));
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

static GPtrArray *
ogmrip_backend_video_command (OGMRipVideo *video, OGMDvdVideoStream *vstream)
{
  static gchar *deinterlacer[] = { "pp=lb", "pp=li", "pp=ci", "pp=md", "pp=fd", "kerndeint" };

  GPtrArray *argv;
  OGMRipScalerType scaler;
  OGMRipDeintType deint;
  GString *options;

  guint rate_numerator, rate_denominator;
  guint output_rate_numerator, output_rate_denominator;
  guint crop_x, crop_y, crop_width, crop_height;
  guint scale_width, scale_height;
  gboolean crop, scale;
  gint step;

  argv = g_ptr_array_new ();

  scaler = ogmrip_video_get_scaler (video);
  deint = ogmrip_video_get_deinterlacer (video);

  g_ptr_array_add (argv, g_strdup ("mencoder"));
  g_ptr_array_add (argv, g_strdup ("-nocache"));
  g_ptr_array_add (argv, g_strdup ("-nosound"));
  g_ptr_array_add (argv, g_strdup ("-noslices"));
  g_ptr_array_add (argv, g_strdup ("-noskip"));
  g_ptr_array_add (argv, g_strdup ("-zoom"));

  g_ptr_array_add (argv, g_strdup ("-skiplimit"));
  g_ptr_array_add (argv, g_strdup ("0"));

  g_ptr_array_add (argv, g_strdup ("-sws"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", MAX (scaler, 0)));

  ogmrip_video_get_crop_size (video, &crop_x, &crop_y, &crop_width, &crop_height);
  crop = (crop_x > 0 || crop_y > 0 || crop_width > 0 || crop_height > 0);

  ogmrip_video_get_scale_size (video, &scale_width, &scale_height);
  scale = (scale_width > 0 || scale_height);

  options = g_string_new (NULL);

  if (ogmrip_video_get_pullup (video))
  {
    if (options->len > 0)
      g_string_append_c (options, ',');
    g_string_append (options, "pullup,softskip");
  }

  if (crop)
  {
    if (options->len > 0)
      g_string_append_c (options, ',');
    g_string_append_printf (options, "crop=%u:%u:%u:%u", crop_width, crop_height, crop_x, crop_y);
  }

  if (deint != OGMRIP_DEINT_NONE)
  {
    if (options->len > 0)
      g_string_append_c (options, ',');
    g_string_append_printf (options, "%s", deinterlacer[deint - 1]);
  }

  if (scale)
  {
    if (options->len > 0)
      g_string_append_c (options, ',');
    g_string_append_printf (options, "scale=%u:%u", scale_width, scale_height);
  }

  if (ogmrip_video_get_denoise (video))
  {
    if (options->len > 0)
      g_string_append_c (options, ',');
    g_string_append (options, "hqdn3d=2:1:2");
  }

  if (options->len > 0)
    g_string_append_c (options, ',');
  g_string_append (options, "harddup");

  if (options->len == 0)
    g_string_free (options, TRUE);
  else
  {
    g_ptr_array_add (argv, g_strdup ("-vf"));
    g_ptr_array_add (argv, g_string_free (options, FALSE));
  }

  ogmdvd_video_stream_get_framerate (vstream, &rate_numerator, &rate_denominator);
  ogmrip_video_get_framerate (video, &output_rate_numerator, &output_rate_denominator);
  step = ogmrip_video_get_framestep (video); 

  if (rate_numerator != output_rate_numerator && rate_denominator != output_rate_denominator * step)
  {
    g_ptr_array_add (argv, g_strdup ("-fps"));
    g_ptr_array_add (argv, g_strdup_printf ("%d/%d", rate_numerator, rate_denominator));

    g_ptr_array_add (argv, g_strdup ("-ofps"));
    g_ptr_array_add (argv, g_strdup_printf ("%d/%d", output_rate_numerator, output_rate_denominator * step));
  }

  return argv;
}

gchar **
ogmrip_backend_xvid_command (OGMRipVideo *video, const gchar *input, const gchar *output, const gchar *log)
{
  OGMDvdVideoStream *vstream;
  GPtrArray *argv;
  GString *options;

  const gchar *device;
  gint vid, pass, start, end;
  gint bitrate;

  g_return_val_if_fail (OGMRIP_IS_VIDEO (video), NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (video));
  g_return_val_if_fail (output != NULL, NULL);

  vstream = ogmrip_codec_get_input (OGMRIP_CODEC (video));
  g_return_val_if_fail (vstream != NULL, NULL);

  pass = ogmrip_video_get_pass (video);
  if (pass > 0 && !log)
    log = ogmrip_video_get_log (video);
  g_return_val_if_fail (pass == 0 || log != NULL, NULL);

  argv = g_ptr_array_new ();
  argv = ogmrip_backend_video_command (video, vstream);

  g_ptr_array_add (argv, g_strdup ("-ovc"));
  g_ptr_array_add (argv, g_strdup ("xvid"));

  options = g_string_new (OGMRIP_XVID_DEFAULT_OPTIONS);

#if MPLAYER_PRE >= 6
  g_string_append (options, ":bvhq=1");

  if (ogmrip_video_get_cartoon (video))
    g_string_append (options, ":cartoon");
#endif /* MPLAYER_PRE */

  if (ogmrip_video_get_qpel (video))
    g_string_append (options, ":qpel");
  if (ogmrip_video_get_turbo (video))
    g_string_append (options, ":turbo");
  if (ogmrip_video_get_trellis (video))
    g_string_append (options, ":trellis");
  if (ogmrip_video_get_grayscale (video))
    g_string_append (options, ":greyscale");

  if (ogmrip_xvid_get_gmc (OGMRIP_XVID (video)))
    g_string_append (options, ":gmc");

  g_string_append_printf (options, ":max_bframes=%d", ogmrip_video_get_max_b_frames (video));

  bitrate = ogmrip_video_get_bitrate (video);
  if (bitrate < 16001)
    g_string_append_printf (options, ":bitrate=%u", bitrate / 1000);
  else
    g_string_append_printf (options, ":bitrate=%u", bitrate);
  
  switch (pass)
  {
    case 1:
      g_string_append_printf (options, ":pass=%u", 1);
      g_ptr_array_add (argv, g_strdup ("-passlogfile"));
      g_ptr_array_add (argv, g_strdup (log));
      break;
    case 2:
      g_string_append_printf (options, ":pass=%u", 2);
      g_ptr_array_add (argv, g_strdup ("-passlogfile"));
      g_ptr_array_add (argv, g_strdup (log));
      break;
    default:
      break;
  }

  g_ptr_array_add (argv, g_strdup ("-xvidencopts"));
  g_ptr_array_add (argv, g_string_free (options, FALSE));

  ogmrip_codec_get_chapters (OGMRIP_CODEC (video), &start, &end);
  g_ptr_array_add (argv, g_strdup ("-chapter"));
  if (end > -1)
    g_ptr_array_add (argv, g_strdup_printf ("%d-%d", start + 1, end + 1));
  else
    g_ptr_array_add (argv, g_strdup_printf ("%d", start + 1));

  g_ptr_array_add (argv, g_strdup ("-o"));
  if (pass == 1)
    g_ptr_array_add (argv, g_strdup ("/dev/null"));
  else
    g_ptr_array_add (argv, g_strdup (output));

  device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (vstream));
  g_ptr_array_add (argv, g_strdup ("-dvd-device"));
  g_ptr_array_add (argv, g_strdup (device));

  vid = ogmdvd_video_stream_get_id (vstream);

#if MPLAYER_MAJOR > 0
  g_ptr_array_add (argv, g_strdup_printf ("dvd://%d", vid + 1));
#else /* MPLAYER_MAJOR */
  g_ptr_array_add (argv, g_strdup ("-dvd"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));
#endif /* MPLAYER_MAJOR */

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

gchar **
ogmrip_backend_lavc_command (OGMRipVideo *video, const gchar *input, const gchar *output, const gchar *log)
{
  OGMDvdVideoStream *vstream;
  GPtrArray *argv;
  GString *options;

  const gchar *device;
  gint vid, pass, start, end;
  guint cmp, precmp, subcmp;
  gint dia, predia;

  g_return_val_if_fail (OGMRIP_IS_VIDEO (video), NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (video));
  g_return_val_if_fail (output != NULL, NULL);

  vstream = ogmrip_codec_get_input (OGMRIP_CODEC (video));
  g_return_val_if_fail (vstream != NULL, NULL);

  pass = ogmrip_video_get_pass (video);
  if (pass > 0 && !log)
    log = ogmrip_video_get_log (video);
  g_return_val_if_fail (pass == 0 || log != NULL, NULL);

  argv = ogmrip_backend_video_command (video, vstream);

  g_ptr_array_add (argv, g_strdup ("-ovc"));
  g_ptr_array_add (argv, g_strdup ("lavc"));

  options = g_string_new (OGMRIP_LAVC_DEFAULT_OPTIONS);

#if MPLAYER_PRE >= 6
  if (ogmrip_video_get_turbo (video))
    g_string_append (options, ":turbo");
#endif /* MPLAYER_PRE */

  if (ogmrip_video_get_qpel (video))
    g_string_append (options, ":qpel");
  if (ogmrip_video_get_4mv (video))
    g_string_append (options, ":v4mv");
  if (ogmrip_video_get_trellis (video))
    g_string_append (options, ":trell:cbp");
  if (ogmrip_video_get_grayscale (video))
    g_string_append (options, ":gray");

  ogmrip_lavc_get_cmp (OGMRIP_LAVC (video), &cmp, &precmp, &subcmp);
  g_string_append_printf (options, ":precmp=%u:subcmp=%u:cmp=%u", precmp, subcmp, cmp);

  ogmrip_lavc_get_dia (OGMRIP_LAVC (video), &dia, &predia);
  g_string_append_printf (options, ":dia=%d:predia=%d", dia, predia);

  g_string_append_printf (options, ":vmax_b_frames=%d", ogmrip_video_get_max_b_frames (video));
  g_string_append_printf (options, ":vbitrate=%u", ogmrip_video_get_bitrate (video));
  
  switch (pass)
  {
    case 1:
      g_string_append_printf (options, ":vpass=%u", 1);
      g_ptr_array_add (argv, g_strdup ("-passlogfile"));
      g_ptr_array_add (argv, g_strdup (log));
      break;
    case 2:
      g_string_append_printf (options, ":vpass=%u", 2);
      g_ptr_array_add (argv, g_strdup ("-passlogfile"));
      g_ptr_array_add (argv, g_strdup (log));
      break;
    default:
      break;
  }

  g_ptr_array_add (argv, g_strdup ("-lavcopts"));
  g_ptr_array_add (argv, g_string_free (options, FALSE));

  ogmrip_codec_get_chapters (OGMRIP_CODEC (video), &start, &end);
  g_ptr_array_add (argv, g_strdup ("-chapter"));
  if (end > -1)
    g_ptr_array_add (argv, g_strdup_printf ("%d-%d", start + 1, end + 1));
  else
    g_ptr_array_add (argv, g_strdup_printf ("%d", start + 1));

  g_ptr_array_add (argv, g_strdup ("-o"));
  if (pass == 1)
    g_ptr_array_add (argv, g_strdup ("/dev/null"));
  else
    g_ptr_array_add (argv, g_strdup (output));

  device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (vstream));
  g_ptr_array_add (argv, g_strdup ("-dvd-device"));
  g_ptr_array_add (argv, g_strdup (device));

  vid = ogmdvd_video_stream_get_id (vstream);

#if MPLAYER_MAJOR > 0
  g_ptr_array_add (argv, g_strdup_printf ("dvd://%d", vid + 1));
#else /* MPLAYER_MAJOR */
  g_ptr_array_add (argv, g_strdup ("-dvd"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));
#endif /* MPLAYER_MAJOR */

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

#ifdef HAVE_X264_SUPPORT
gchar **
ogmrip_backend_x264_command (OGMRipVideo *video, const gchar *input, const gchar *output, const gchar *log)
{
  OGMDvdVideoStream *vstream;
  GPtrArray *argv;
  GString *options;

  const gchar *device;
  gint vid, pass, start, end/*, subq*/;

  g_return_val_if_fail (OGMRIP_IS_VIDEO (video), NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (video));
  g_return_val_if_fail (output != NULL, NULL);

  vstream = ogmrip_codec_get_input (OGMRIP_CODEC (video));
  g_return_val_if_fail (vstream != NULL, NULL);

  pass = ogmrip_video_get_pass (video);
  if (pass > 0 && !log)
    log = ogmrip_video_get_log (video);
  g_return_val_if_fail (pass == 0 || log != NULL, NULL);

  argv = ogmrip_backend_video_command (video, vstream);

  g_ptr_array_add (argv, g_strdup ("-ovc"));
  g_ptr_array_add (argv, g_strdup ("x264"));

  options = g_string_new (OGMRIP_X264_DEFAULT_OPTIONS);

  g_string_append_printf (options, ":bitrate=%u", ogmrip_video_get_bitrate (video));
  
  switch (pass)
  {
    case 0:
      break;
    case 1:
      g_string_append_printf (options, ":pass=%u", 1);
      g_ptr_array_add (argv, g_strdup ("-passlogfile"));
      g_ptr_array_add (argv, g_strdup (log));
      break;
    case 2:
      g_string_append_printf (options, ":pass=%u", 2);
      g_ptr_array_add (argv, g_strdup ("-passlogfile"));
      g_ptr_array_add (argv, g_strdup (log));
      break;
    default:
      break;
  }
/*
  if (ogmrip_video_get_turbo (video))
    subq = pass == 2 ? 3 : 1;
  else
    subq = pass == 2 ? 6 : 3;

  if (ogmrip_video_get_4mv (video))
  {
    g_string_append (options, ":4x4mv");
    subq = MAX (subq, 3);
  }

  g_string_append_printf (options, ":subq=%d", subq);
*/
  g_ptr_array_add (argv, g_strdup ("-x264encopts"));
  g_ptr_array_add (argv, g_string_free (options, FALSE));

  ogmrip_codec_get_chapters (OGMRIP_CODEC (video), &start, &end);
  g_ptr_array_add (argv, g_strdup ("-chapter"));
  if (end > -1)
    g_ptr_array_add (argv, g_strdup_printf ("%d-%d", start + 1, end + 1));
  else
    g_ptr_array_add (argv, g_strdup_printf ("%d", start + 1));

  g_ptr_array_add (argv, g_strdup ("-o"));
  if (pass == 1)
    g_ptr_array_add (argv, g_strdup ("/dev/null"));
  else
    g_ptr_array_add (argv, g_strdup (output));

  device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (vstream));
  g_ptr_array_add (argv, g_strdup ("-dvd-device"));
  g_ptr_array_add (argv, g_strdup (device));

  vid = ogmdvd_video_stream_get_id (vstream);

#if MPLAYER_MAJOR > 0
  g_ptr_array_add (argv, g_strdup_printf ("dvd://%d", vid + 1));
#else /* MPLAYER_MAJOR */
  g_ptr_array_add (argv, g_strdup ("-dvd"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));
#endif /* MPLAYER_MAJOR */

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}
#endif /* HAVE_X264_SUPPORT */

#ifdef OGMRIP_THEORA_SUPPORT
gchar **
ogmrip_backend_yuv4mpeg_command (OGMRipVideo *video, const gchar *input, const gchar *output, const gchar *log)
{
  OGMDvdVideoStream *vstream;
  OGMRipScalerType scaler;
  GPtrArray *argv;

  const gchar *device;
  gint vid, start, end;
  guint crop_x, crop_y, crop_width, crop_height;
  guint scale_width, scale_height;
  gboolean crop, scale;

  g_return_val_if_fail (OGMRIP_IS_VIDEO (video), NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (video));
  g_return_val_if_fail (output != NULL, NULL);

  vstream = ogmrip_codec_get_input (OGMRIP_CODEC (video));
  g_return_val_if_fail (vstream != NULL, NULL);

  argv = g_ptr_array_new ();

  g_ptr_array_add (argv, g_strdup ("mplayer"));
  g_ptr_array_add (argv, g_strdup ("-nocache"));
  g_ptr_array_add (argv, g_strdup ("-noframedrop"));
  g_ptr_array_add (argv, g_strdup ("-nosound"));
  g_ptr_array_add (argv, g_strdup ("-noslices"));
  g_ptr_array_add (argv, g_strdup ("-zoom"));

  scaler = ogmrip_video_get_scaler (video);

  g_ptr_array_add (argv, g_strdup ("-sws"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", MAX (scaler, 0)));

  ogmrip_video_get_crop_size (video, &crop_x, &crop_y, &crop_width, &crop_height);
  crop = (crop_x > 0 || crop_y > 0 || crop_width > 0 || crop_height > 0);

  ogmrip_video_get_scale_size (video, &scale_width, &scale_height);
  scale = (scale_width > 0 || scale_height);

  if (scale || crop)
  {
    GString *options;
    
    options = g_string_new (NULL);

    if (crop)
      g_string_append_printf (options, "crop=%u:%u:%u:%u", crop_width, crop_height, crop_x, crop_y);

    if (crop && scale)
      g_string_append_c (options, ',');

    if (scale)
      g_string_append_printf (options, "scale=%u:%u", scale_width, scale_height);

    g_ptr_array_add (argv, g_strdup ("-vf"));
    g_ptr_array_add (argv, g_string_free (options, FALSE));
  }

  g_ptr_array_add (argv, g_strdup ("-vo"));
#if MPLAYER_PRE >= 6
  g_ptr_array_add (argv, g_strdup_printf ("yuv4mpeg:file=%s", output));
#else /* MPLAYER_PRE */
  g_ptr_array_add (argv, g_strdup ("yuv4mpeg"));
#endif /* MPLAYER_PRE */

  ogmrip_codec_get_chapters (OGMRIP_CODEC (video), &start, &end);

  g_ptr_array_add (argv, g_strdup ("-chapter"));
  if (end > -1)
    g_ptr_array_add (argv, g_strdup_printf ("%d-%d", start + 1, end + 1));
  else
    g_ptr_array_add (argv, g_strdup_printf ("%d", start + 1));

  device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (vstream));
  g_ptr_array_add (argv, g_strdup ("-dvd-device"));
  g_ptr_array_add (argv, g_strdup (device));

  vid = ogmdvd_video_stream_get_id (vstream);

#if MPLAYER_MAJOR > 0
  g_ptr_array_add (argv, g_strdup_printf ("dvd://%d", vid + 1));
#else /* MPLAYER_MAJOR */
  g_ptr_array_add (argv, g_strdup ("-dvd"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));
#endif /* MPLAYER_MAJOR */

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

gchar **
ogmrip_backend_theora_command (OGMRipVideo *video, const gchar *input, const gchar *output, const gchar *log)
{
  GPtrArray *argv;

  g_return_val_if_fail (OGMRIP_IS_VIDEO (video), NULL);
  g_return_val_if_fail (input != NULL, NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (video));
  g_return_val_if_fail (output != NULL, NULL);

  argv = g_ptr_array_new ();

  g_ptr_array_add (argv, g_strdup ("theora-encoder"));

  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup (output));

  g_ptr_array_add (argv, g_strdup ("-V"));
  g_ptr_array_add (argv, g_strdup_printf ("%u", ogmrip_video_get_bitrate (video)));

  g_ptr_array_add (argv, g_strdup (input));

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}
#endif /* OGMRIP_THEORA_SUPPORT */

gchar **
ogmrip_backend_crop_command (OGMRipVideo *video, const gchar *input, const gchar *output)
{
  OGMDvdTime time_;
  OGMDvdVideoStream *vstream;
  GPtrArray *argv;

  const gchar *device;
  gint vid;

  g_return_val_if_fail (OGMRIP_IS_VIDEO (video), NULL);

  vstream = ogmrip_codec_get_input (OGMRIP_CODEC (video));
  g_return_val_if_fail (vstream != NULL, NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("mencoder"));
  g_ptr_array_add (argv, g_strdup ("-nosound"));
  g_ptr_array_add (argv, g_strdup ("-nocache"));

  g_ptr_array_add (argv, g_strdup ("-ovc"));
  g_ptr_array_add (argv, g_strdup ("lavc"));
  g_ptr_array_add (argv, g_strdup ("-vf"));
  g_ptr_array_add (argv, g_strdup ("cropdetect"));

  ogmrip_codec_get_length (OGMRIP_CODEC (video), &time_);

  g_ptr_array_add (argv, g_strdup ("-ss"));
  g_ptr_array_add (argv, g_strdup_printf ("%02u:%02u:%02u", 
        time_.hour / 4, time_.min / 4 + (time_.hour % 4) * 15, time_.sec / 4 + (time_.min % 4) * 15));

  g_ptr_array_add (argv, g_strdup ("-frames"));
  g_ptr_array_add (argv, g_strdup ("30"));
  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup ("/dev/null"));

  device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (vstream));
  g_ptr_array_add (argv, g_strdup ("-dvd-device"));
  g_ptr_array_add (argv, g_strdup (device));

  vid = ogmdvd_video_stream_get_id (vstream);

#if MPLAYER_MAJOR > 0
  g_ptr_array_add (argv, g_strdup_printf ("dvd://%d", vid + 1));
#else /* MPLAYER_MAJOR */
  g_ptr_array_add (argv, g_strdup ("-dvd"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));
#endif /* MPLAYER_MAJOR */

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

static void
ogmrip_backend_merge_foreach_audio (OGMRipContainer *ogg, 
    const gchar *input, gint language, GPtrArray *argv)
{
  if (language > -1)
  {
    g_ptr_array_add (argv, g_strdup ("-c"));
    g_ptr_array_add (argv, g_strdup_printf ("'LANGUAGE=%s'", 
          g_strdup (ogmdvd_get_language_name (language))));
    g_ptr_array_add (argv, g_strdup ("--novideo"));
  }

  g_ptr_array_add (argv, g_strdup (input));
}

static void
ogmrip_backend_merge_foreach_text (OGMRipContainer *ogg, 
    const gchar *input, gint language, GPtrArray *argv)
{
  if (language > -1)
  {
    g_ptr_array_add (argv, g_strdup ("-c"));
    g_ptr_array_add (argv, g_strdup_printf ("'LANGUAGE=%s'",
          g_strdup (ogmdvd_get_language_name (language))));
  }

  g_ptr_array_add (argv, g_strdup (input));
}

gchar **
ogmrip_backend_ogg_merge_command (OGMRipContainer *ogg, const gchar *output)
{
  GPtrArray *argv;
  const gchar *title;
  const gchar *video;

  g_return_val_if_fail (OGMRIP_IS_OGG (ogg), NULL);

  if (!output)
    output = ogmrip_container_get_output (ogg);
  g_return_val_if_fail (output != NULL, NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("ogmmerge"));

  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup (output));

  title = ogmrip_container_get_title (ogg);
  if (title)
  {
    g_ptr_array_add (argv, g_strdup ("-c"));
    g_ptr_array_add (argv, g_strdup_printf ("'TITLE=%s'", title));
    g_ptr_array_add (argv, g_strdup ("--noaudio"));
  }

  video = ogmrip_container_get_video (ogg);
  g_ptr_array_add (argv, g_strdup (video));

  ogmrip_container_foreach_audio (ogg, (OGMRipContainerFunc) ogmrip_backend_merge_foreach_audio, argv);
  ogmrip_container_foreach_chapter (ogg, (OGMRipContainerFunc) ogmrip_backend_merge_foreach_text, argv);
  ogmrip_container_foreach_subp (ogg, (OGMRipContainerFunc) ogmrip_backend_merge_foreach_text, argv);

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

gchar **
ogmrip_backend_ogg_split_command (OGMRipContainer *ogg, const gchar *input)
{
  GPtrArray *argv;
  const gchar *output;
  gint tsize;

  g_return_val_if_fail (OGMRIP_IS_CONTAINER (ogg), NULL);
  g_return_val_if_fail (input && *input, NULL);

  output = ogmrip_container_get_output (ogg);
  g_return_val_if_fail (output && *output, NULL);

  tsize = ogmrip_container_get_tsize (OGMRIP_CONTAINER (ogg));
  g_return_val_if_fail (tsize > 0, NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("ogmsplit"));
  g_ptr_array_add (argv, g_strdup ("--frontend"));
  g_ptr_array_add (argv, g_strdup ("-s"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", tsize));
  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup (output));
  g_ptr_array_add (argv, g_strdup (input));
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

#ifdef HAVE_MKV_SUPPORT
static void
ogmrip_backend_matroska_foreach_audio (OGMRipContainer *matroska, 
    const gchar *input, gint language, GPtrArray *argv)
{
  if (language > -1)
  {
    const gchar *iso639_2;

    iso639_2 = ogmdvd_get_language_iso639_2 (language);
    if (iso639_2)
    {
      g_ptr_array_add (argv, g_strdup ("--language"));
      g_ptr_array_add (argv, g_strconcat ("0:", iso639_2, NULL));
      g_ptr_array_add (argv, g_strdup ("--language"));
      g_ptr_array_add (argv, g_strconcat ("1:", iso639_2, NULL));
    }
  }
/*
  g_ptr_array_add (argv, g_strdup ("-a"));
  g_ptr_array_add (argv, g_strdup ("0"));
*/
  g_ptr_array_add (argv, g_strdup ("-D"));
  g_ptr_array_add (argv, g_strdup ("-S"));
  g_ptr_array_add (argv, g_strdup (input));
}

static void
ogmrip_backend_matroska_foreach_chapters (OGMRipContainer *matroska, 
    const gchar *input, gint language, GPtrArray *argv)
{
  if (language > -1)
  {
    const gchar *iso639_2;

    iso639_2 = ogmdvd_get_language_iso639_2 (language);
    if (iso639_2)
    {
      g_ptr_array_add (argv, g_strdup ("--chapter-language"));
      g_ptr_array_add (argv, g_strdup (iso639_2));
    }
  }

  g_ptr_array_add (argv, g_strdup ("--chapter-charset"));
  g_ptr_array_add (argv, g_strdup ("UTF-8"));
  g_ptr_array_add (argv, g_strdup ("--chapters"));
  g_ptr_array_add (argv, g_strdup (input));
}

static void
ogmrip_backend_matroska_foreach_subp (OGMRipContainer *matroska, 
    const gchar *input, gint language, GPtrArray *argv)
{
  if (language > -1)
  {
    const gchar *iso639_2;

    iso639_2 = ogmdvd_get_language_iso639_2 (language);
    if (iso639_2)
    {
      g_ptr_array_add (argv, g_strdup ("--language"));
      g_ptr_array_add (argv, g_strconcat ("0:", iso639_2, NULL));
    }
  }

  g_ptr_array_add (argv, g_strdup ("--sub-charset"));
  g_ptr_array_add (argv, g_strdup ("0:UTF-8"));
  g_ptr_array_add (argv, g_strdup ("-s"));
  g_ptr_array_add (argv, g_strdup ("0"));
  g_ptr_array_add (argv, g_strdup ("-D"));
  g_ptr_array_add (argv, g_strdup ("-A"));
  g_ptr_array_add (argv, g_strdup (input));
}

gchar **
ogmrip_backend_matroska_command (OGMRipContainer *matroska)
{
  GPtrArray *argv;
  const gchar *output;
  const gchar *title;
  const gchar *video;
  gint tsize;

  g_return_val_if_fail (OGMRIP_IS_MATROSKA (matroska), NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("mkvmerge"));

  output = ogmrip_container_get_output (matroska);
  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup (output));

  g_ptr_array_add (argv, g_strdup ("--command-line-charset"));
  g_ptr_array_add (argv, g_strdup ("UTF-8"));

  video = ogmrip_container_get_video (matroska);
  g_ptr_array_add (argv, g_strdup ("-d"));
  g_ptr_array_add (argv, g_strdup ("0"));
  g_ptr_array_add (argv, g_strdup ("-A"));
  g_ptr_array_add (argv, g_strdup ("-S"));
  g_ptr_array_add (argv, g_strdup (video));

  ogmrip_container_foreach_audio (matroska, (OGMRipContainerFunc) ogmrip_backend_matroska_foreach_audio, argv);
  ogmrip_container_foreach_subp (matroska, (OGMRipContainerFunc) ogmrip_backend_matroska_foreach_subp, argv);
  ogmrip_container_foreach_chapter (matroska, (OGMRipContainerFunc) ogmrip_backend_matroska_foreach_chapters, argv);

  title = ogmrip_container_get_title (matroska);
  if (title)
  {
    g_ptr_array_add (argv, g_strdup ("--title"));
    g_ptr_array_add (argv, g_strdup_printf ("'%s'", title));
  }

  if (ogmrip_container_get_tnumber (matroska) > 1)
  {
    tsize = ogmrip_container_get_tsize (matroska);
    g_ptr_array_add (argv, g_strdup ("--split"));
    g_ptr_array_add (argv, g_strdup_printf ("%dM", tsize));
  }

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}
#endif /* HAVE_MKV_SUPPORT */

static void
ogmrip_backend_avi_foreach (OGMRipContainer *avi, 
    const gchar *input, gint language, GPtrArray *argv)
{
  gchar *basename;

  basename = g_path_get_basename (input);

  if (strncmp (basename, "frameno", 7) != 0)
  {
    g_ptr_array_add (argv, g_strdup ("-audiofile"));
    g_ptr_array_add (argv, g_strdup (input));
  }
  else
  {
/*
#if MPLAYER_PRE >= 8
    g_ptr_array_add (argv, g_strdup ("-framenofile"));
    g_ptr_array_add (argv, g_strdup (input));
#else
*/
    gchar *dirname, *new_name;

    dirname = g_path_get_dirname (input);
    new_name = g_build_filename (dirname, "frameno.avi", NULL);
    g_rename (input, new_name);
    strcpy ((gchar *) input, new_name);
/*
#endif
*/
  }

  g_free (basename);
}

gchar **
ogmrip_backend_avi_command (OGMRipContainer *avi)
{
  GPtrArray *argv;
  const gchar *output;
  const gchar *video;

  g_return_val_if_fail (OGMRIP_IS_AVI (avi), NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("mencoder"));
  g_ptr_array_add (argv, g_strdup ("-nocache"));
  g_ptr_array_add (argv, g_strdup ("-noskip"));
  g_ptr_array_add (argv, g_strdup ("-skiplimit"));
  g_ptr_array_add (argv, g_strdup ("0"));
  g_ptr_array_add (argv, g_strdup ("-ovc"));
  g_ptr_array_add (argv, g_strdup ("copy"));
  g_ptr_array_add (argv, g_strdup ("-oac"));
  g_ptr_array_add (argv, g_strdup ("copy"));

  output = ogmrip_container_get_output (avi);
  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup (output));

  ogmrip_container_foreach_audio (avi, 
      (OGMRipContainerFunc) ogmrip_backend_avi_foreach, argv);

  video = ogmrip_container_get_video (avi);
  g_ptr_array_add (argv, g_strdup (video));

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

gchar **
ogmrip_backend_vobsub_command (OGMRipSubp *subp, const gchar *input, const gchar *output)
{
  OGMDvdVideoStream *vstream;
  OGMDvdSubpStream *sstream;

  GPtrArray *argv;
  const gchar *device;
  gint vid, sid, start, end;

  g_return_val_if_fail (OGMRIP_IS_SUBP (subp), NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (subp));
  g_return_val_if_fail (output != NULL, NULL);

  vstream = ogmrip_codec_get_input (OGMRIP_CODEC (subp));
  g_return_val_if_fail (vstream != NULL, NULL);

  sstream = ogmrip_subp_get_dvd_subp_stream (OGMRIP_SUBP (subp));
  g_return_val_if_fail (sstream != NULL, NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("mencoder"));
  g_ptr_array_add (argv, g_strdup ("-nocache"));
  g_ptr_array_add (argv, g_strdup ("-nosound"));
  g_ptr_array_add (argv, g_strdup ("-ovc"));
  g_ptr_array_add (argv, g_strdup ("copy"));
  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup ("/dev/null"));

  sid = ogmdvd_subp_stream_get_id (sstream);
  g_ptr_array_add (argv, g_strdup ("-vobsubout"));
  g_ptr_array_add (argv, g_strdup (output));
  g_ptr_array_add (argv, g_strdup ("-vobsuboutindex"));
  g_ptr_array_add (argv, g_strdup ("0"));
  g_ptr_array_add (argv, g_strdup ("-sid"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", sid));

  ogmrip_codec_get_chapters (OGMRIP_CODEC (subp), &start, &end);
  g_ptr_array_add (argv, g_strdup ("-chapter"));
  if (end > -1)
    g_ptr_array_add (argv, g_strdup_printf ("%d-%d", start + 1, end + 1));
  else
    g_ptr_array_add (argv, g_strdup_printf ("%d", start + 1));

  device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (vstream));
  g_ptr_array_add (argv, g_strdup ("-dvd-device"));
  g_ptr_array_add (argv, g_strdup (device));

  vid = ogmdvd_video_stream_get_id (vstream);

#if MPLAYER_MAJOR > 0
  g_ptr_array_add (argv, g_strdup_printf ("dvd://%d", vid + 1));
#else /* MPLAYER_MAJOR */
  g_ptr_array_add (argv, g_strdup ("-dvd"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));
#endif /* MPLAYER_MAJOR */

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

#ifdef HAVE_SRT_SUPPORT
gchar **
ogmrip_backend_pgm_command (OGMRipSubp *subp, const gchar *input, const gchar *output)
{
  GPtrArray *argv;

  g_return_val_if_fail (OGMRIP_IS_SUBP (subp), NULL);
  g_return_val_if_fail (input != NULL, NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("subp2pgm"));
  g_ptr_array_add (argv, g_strdup (input));
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

gchar **
ogmrip_backend_ocr_command (OGMRipSubp *subp, const gchar *input, const gchar *output)
{
  GPtrArray *argv;

  g_return_val_if_fail (OGMRIP_IS_SUBP (subp), NULL);
  g_return_val_if_fail (input != NULL, NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("gocr"));
  g_ptr_array_add (argv, g_strdup ("-v"));
  g_ptr_array_add (argv, g_strdup ("1"));
  g_ptr_array_add (argv, g_strdup ("-f"));
  g_ptr_array_add (argv, g_strdup ("UTF8"));
  g_ptr_array_add (argv, g_strdup ("-m"));
  g_ptr_array_add (argv, g_strdup ("4"));
  g_ptr_array_add (argv, g_strdup ("-m"));
  g_ptr_array_add (argv, g_strdup ("64"));
  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strconcat (input, ".txt", NULL));
  g_ptr_array_add (argv, g_strdup (input));
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

gchar **
ogmrip_backend_srt_command (OGMRipSubp *subp, const gchar *input, const gchar *output)
{
  GPtrArray *argv;

  g_return_val_if_fail (OGMRIP_IS_SUBP (subp), NULL);
  g_return_val_if_fail (input != NULL, NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (subp));
  g_return_val_if_fail (output != NULL, NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("srttool"));
  g_ptr_array_add (argv, g_strdup ("-s"));
  g_ptr_array_add (argv, g_strdup ("-w"));
  g_ptr_array_add (argv, g_strdup ("-i"));
  g_ptr_array_add (argv, g_strdup (input));
  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup (output));
  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}
#endif /* HAVE_SRT_SUPPORT */

gchar **
ogmrip_backend_dvdcpy_command (OGMRipDvdCpy *dvdcpy, const gchar *input, const gchar *output)
{
  OGMDvdVideoStream *vstream;
  GPtrArray *argv;

  const gchar *device;
  gint vid;

  g_return_val_if_fail (OGMRIP_IS_DVDCPY (dvdcpy), NULL);

  if (!output)
    output = ogmrip_codec_get_output (OGMRIP_CODEC (dvdcpy));
  g_return_val_if_fail (output != NULL, NULL);

  vstream = ogmrip_codec_get_input (OGMRIP_CODEC (dvdcpy));
  g_return_val_if_fail (vstream != NULL, NULL);

  argv = g_ptr_array_new ();
  g_ptr_array_add (argv, g_strdup ("dvdcpy"));
  g_ptr_array_add (argv, g_strdup ("-o"));
  g_ptr_array_add (argv, g_strdup (output));
  g_ptr_array_add (argv, g_strdup ("-m"));

  vid = ogmdvd_video_stream_get_id (vstream);
  g_ptr_array_add (argv, g_strdup ("-t"));
  g_ptr_array_add (argv, g_strdup_printf ("%d", vid + 1));

  device = ogmdvd_get_device (ogmdvd_video_stream_get_dvd (vstream));
  g_ptr_array_add (argv, g_strdup (device));

  g_ptr_array_add (argv, NULL);

  return (gchar **) g_ptr_array_free (argv, FALSE);
}

