/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-mkv.h"
#include "ogmrip-backend.h"
#include "ogmsp-enums.h"
#include "ogmsp-exec.h"

#include <string.h>

static gint   ogmrip_matroska_run               (OGMSpSpawn      *spawn);
static gint64 ogmrip_matroska_get_overhead_size (OGMRipContainer *container,
                                                 guint64         size,
                                                 gulong          length);

G_DEFINE_TYPE (OGMRipMatroska, ogmrip_matroska, OGMRIP_TYPE_CONTAINER)

static void
ogmrip_matroska_class_init (OGMRipMatroskaClass *klass)
{
  OGMSpSpawnClass *spawn_class;
  OGMRipContainerClass *container_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);
  container_class = OGMRIP_CONTAINER_CLASS (klass);

  spawn_class->run = ogmrip_matroska_run;
  container_class->get_overhead_size = ogmrip_matroska_get_overhead_size;
}

static void
ogmrip_matroska_init (OGMRipMatroska *matroska)
{
}

static gint
ogmrip_matroska_run (OGMSpSpawn *spawn)
{
  OGMSpSpawn *child;
  gchar **argv;
  gint result;

  argv = ogmrip_backend_matroska_command (OGMRIP_CONTAINER (spawn));
  if (!argv)
    return OGMSP_RESULT_ERROR;

  child = ogmsp_exec_newv (argv);
  ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_matroska_watch, spawn, TRUE, FALSE, FALSE);
  ogmsp_container_add (OGMSP_CONTAINER (spawn), child);
  g_object_unref (child);

  result = OGMSP_SPAWN_CLASS (ogmrip_matroska_parent_class)->run (spawn);

  ogmsp_container_remove (OGMSP_CONTAINER (spawn), child);

  return result;
}

OGMSpSpawn *
ogmrip_matroska_new (const gchar *output)
{
  OGMRipMatroska *matroska;

  g_return_val_if_fail (output && *output, NULL);

  matroska = g_object_new (OGMRIP_TYPE_MATROSKA, NULL);
  ogmrip_container_set_output (OGMRIP_CONTAINER (matroska), output);

  return OGMSP_SPAWN (matroska);
}

static gint64
ogmrip_matroska_get_overhead_size (OGMRipContainer *container, guint64 size, gulong length)
{
  g_return_val_if_fail (OGMRIP_IS_MATROSKA (container), -1);

  /* Video Overhead */
  size = (gint64) 13 * (gint64) length;

  /* Audio Overhead */
  size += (gint64) length * (gint64) ogmrip_container_count_audio (container) * (gint64) 2;

  /* Subp Overhead */
  size += (gint64) ogmrip_container_count_subp (container) * (gint64) 10;

  return size;
}

