/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-subp.h"

#define OGMRIP_SUBP_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_SUBP, OGMRipSubpPriv))

struct _OGMRipSubpPriv
{
  OGMDvdSubpStream *stream;
};

static void ogmrip_subp_finalize (GObject *gobject);

G_DEFINE_ABSTRACT_TYPE (OGMRipSubp, ogmrip_subp, OGMRIP_TYPE_CODEC)

static void
ogmrip_subp_class_init (OGMRipSubpClass *klass)
{
  GObjectClass *gobject_class;

  gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->finalize = ogmrip_subp_finalize;

  g_type_class_add_private (klass, sizeof (OGMRipSubpPriv));
}

static void
ogmrip_subp_init (OGMRipSubp *subp)
{
  subp->priv = OGMRIP_SUBP_GET_PRIVATE (subp);
}

static void
ogmrip_subp_finalize (GObject *gobject)
{
  OGMRipSubp *subp;

  subp = OGMRIP_SUBP (gobject);
  if (subp->priv->stream)
    ogmdvd_subp_stream_unref (subp->priv->stream);

  G_OBJECT_CLASS (ogmrip_subp_parent_class)->finalize (gobject);
}

void
ogmrip_subp_set_dvd_subp_stream (OGMRipSubp *subp, OGMDvdSubpStream *stream)
{
  g_return_if_fail (OGMRIP_IS_SUBP (subp));
  g_return_if_fail (stream != NULL);

  ogmdvd_subp_stream_ref (stream);

  if (subp->priv->stream)
    ogmdvd_subp_stream_unref (subp->priv->stream);

  subp->priv->stream = stream;

  ogmrip_codec_set_input (OGMRIP_CODEC (subp), 
      ogmdvd_subp_stream_get_video_stream (stream));
}

OGMDvdSubpStream *
ogmrip_subp_get_dvd_subp_stream (OGMRipSubp *subp)
{
  g_return_val_if_fail (OGMRIP_IS_SUBP (subp), NULL);

  return subp->priv->stream;
}

