/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2005 Olivier Rolland <billl@esil.univ-mrs.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-vorbis.h"
#include "ogmrip-backend.h"
#include "ogmrip-tools.h"

#include "ogmsp-pipeline.h"
#include "ogmsp-enums.h"
#include "ogmsp-exec.h"

#include <glib/gstdio.h>
#include <unistd.h>

static gint ogmrip_vorbis_run (OGMSpSpawn *spawn);

G_DEFINE_TYPE (OGMRipVorbis, ogmrip_vorbis, OGMRIP_TYPE_AUDIO)

static void
ogmrip_vorbis_class_init (OGMRipVorbisClass *klass)
{
  OGMSpSpawnClass *spawn_class;

  spawn_class = OGMSP_SPAWN_CLASS (klass);

  spawn_class->run = ogmrip_vorbis_run;
}

static void
ogmrip_vorbis_init (OGMRipVorbis *vorbis)
{
}

static gint
ogmrip_vorbis_run (OGMSpSpawn *spawn)
{
  OGMSpSpawn *pipeline;
  OGMSpSpawn *child;
  gchar **argv, *fifo;
  gint result;

  result = OGMSP_RESULT_ERROR;

  fifo = ogmrip_mkfifo ("fifo.XXXXXX", NULL);

  pipeline = ogmsp_pipeline_new ();
  ogmsp_container_add (OGMSP_CONTAINER (spawn), pipeline);
  g_object_unref (pipeline);

  argv = ogmrip_backend_wav_command (OGMRIP_AUDIO (spawn), FALSE, NULL, fifo);
  if (argv)
  {
    child = ogmsp_exec_newv (argv);
    ogmsp_exec_add_watch_full (OGMSP_EXEC (child), (OGMSpWatch) ogmrip_backend_wav_watch, spawn, TRUE, FALSE, FALSE);
    ogmsp_container_add (OGMSP_CONTAINER (pipeline), child);
    g_object_unref (child);

    argv = ogmrip_backend_vorbis_command (OGMRIP_AUDIO (spawn), fifo, NULL);
    if (argv)
    {
      child = ogmsp_exec_newv (argv);
      ogmsp_container_add (OGMSP_CONTAINER (pipeline), child);
      g_object_unref (child);

      result = OGMSP_SPAWN_CLASS (ogmrip_vorbis_parent_class)->run (spawn);
    }
  }

  ogmsp_container_remove (OGMSP_CONTAINER (spawn), pipeline);

  if (g_file_test (fifo, G_FILE_TEST_EXISTS))
    g_unlink (fifo);
  g_free (fifo);

  return result;
}

OGMSpSpawn *
ogmrip_vorbis_new (OGMDvdAudioStream *audio, const gchar *output)
{
  OGMSpSpawn *spawn;

  spawn = g_object_new (OGMRIP_TYPE_VORBIS, NULL);
  ogmrip_codec_set_output (OGMRIP_CODEC (spawn), output);
  ogmrip_audio_set_dvd_audio_stream (OGMRIP_AUDIO (spawn), audio);

  return spawn;
}

