/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004-2005 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "config.h"

#include <string.h>
#include <glib/gi18n.h>

#include "ogmrip-dvd-dialog.h"
#include "ogmrip-dvd-chooser.h"

#define OGMRIP_DVD_DIALOG_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_DVD_DIALOG, OGMRipDvdDialogPriv))

enum
{
  EJECT,
  LAST_SIGNAL
};

struct OGMRipDvdDialogPriv
{
  GtkWidget *chooser;

  GtkWidget *eject_button;
  GtkWidget *load_button;
};

static void ogmrip_dvd_dialog_init (OGMRipDvdDialog *dialog);

static int signals[LAST_SIGNAL] = { 0 };

G_DEFINE_TYPE (OGMRipDvdDialog, ogmrip_dvd_dialog, GTK_TYPE_DIALOG);

static void
ogmrip_dvd_dialog_eject_clicked (GtkDialog *dialog, gint response_id)
{
  const NautilusBurnDrive *drive;
    
  drive = ogmrip_dvd_chooser_get_drive (OGMRIP_DVD_CHOOSER (OGMRIP_DVD_DIALOG (dialog)->priv->chooser));
  if (drive)
  {
    g_signal_emit (dialog, signals[EJECT], 0, NULL);
    nautilus_burn_drive_eject ((NautilusBurnDrive *) drive);
  }
}

static void
ogmrip_dvd_dialog_class_init (OGMRipDvdDialogClass *klass)
{
  GObjectClass *object_class;

  object_class = G_OBJECT_CLASS (klass);

  signals[EJECT] = g_signal_new ("eject", G_TYPE_FROM_CLASS (object_class), 
      G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (OGMRipDvdDialogClass, eject), 
      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);

  g_type_class_add_private (klass, sizeof (OGMRipDvdDialogPriv));
}

static void
ogmrip_dvd_dialog_device_changed (OGMRipDvdDialog *dialog, const gchar *device, GtkWidget *chooser)
{
  gtk_widget_set_sensitive (dialog->priv->load_button, device != NULL);
  gtk_widget_set_sensitive (dialog->priv->eject_button, device != NULL);
}

static void
ogmrip_dvd_dialog_init (OGMRipDvdDialog *dialog)
{
  const NautilusBurnDrive *drive;

  GtkWidget *alignment;
  GtkWidget *image;
  GtkWidget *label;
  GtkWidget *hbox;
  GtkWidget *vbox;

  dialog->priv = OGMRIP_DVD_DIALOG_GET_PRIVATE (dialog);

  gtk_window_set_title (GTK_WINDOW (dialog), _("Open DVD Disk"));
  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);

  dialog->priv->eject_button = gtk_button_new ();
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), dialog->priv->eject_button, FALSE, FALSE, 0);
  gtk_widget_show (dialog->priv->eject_button);

  g_signal_connect_swapped (dialog->priv->eject_button, "clicked", G_CALLBACK (ogmrip_dvd_dialog_eject_clicked), dialog);

  alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
  gtk_container_add (GTK_CONTAINER (dialog->priv->eject_button), alignment);
  gtk_widget_show (alignment);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (alignment), hbox);
  gtk_widget_show (hbox);

  image = gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
  gtk_widget_show (image);

  label = gtk_label_new (_("_Eject"));
  gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  dialog->priv->load_button = gtk_button_new ();
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), dialog->priv->load_button, GTK_RESPONSE_OK);
  gtk_widget_show (dialog->priv->load_button);

  alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
  gtk_container_add (GTK_CONTAINER (dialog->priv->load_button), alignment);
  gtk_widget_show (alignment);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (alignment), hbox);
  gtk_widget_show (hbox);

  image = gtk_image_new_from_stock (GTK_STOCK_CDROM, GTK_ICON_SIZE_BUTTON);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
  gtk_widget_show (image);

  label = gtk_label_new (_("_Load"));
  gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  vbox = gtk_vbox_new (FALSE, 6);
  gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, TRUE, TRUE, 0);
  gtk_widget_show (vbox);

  label = gtk_label_new (_("<b>Select _DVD Drive:</b>"));
  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
  gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  dialog->priv->chooser = ogmrip_dvd_chooser_new ();
  gtk_box_pack_start (GTK_BOX (vbox), dialog->priv->chooser, TRUE, TRUE, 0);
  gtk_widget_show (dialog->priv->chooser);

  g_signal_connect_swapped (dialog->priv->chooser, "device-changed", 
      G_CALLBACK (ogmrip_dvd_dialog_device_changed), dialog);

  drive = ogmrip_dvd_chooser_get_drive (OGMRIP_DVD_CHOOSER (dialog->priv->chooser));
  gtk_widget_set_sensitive (dialog->priv->eject_button, drive != NULL);
  gtk_widget_set_sensitive (dialog->priv->load_button, drive != NULL);
}

GtkWidget *
ogmrip_dvd_dialog_new (void)
{
  GtkWidget *widget;

  widget = g_object_new (OGMRIP_TYPE_DVD_DIALOG, NULL);

  return widget;
}

const NautilusBurnDrive *
ogmrip_dvd_dialog_get_drive (OGMRipDvdDialog *dialog)
{
  g_return_val_if_fail (OGMRIP_IS_DVD_DIALOG (dialog), NULL);

  return ogmrip_dvd_chooser_get_drive (OGMRIP_DVD_CHOOSER (dialog->priv->chooser));
}

