/* OGMRip - A DVD Encoder for GNOME
 * Copyright (C) 2004-2005 Olivier Rolland <billl@users.sf.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef __OGMRIP_HELPER_H__
#define __OGMRIP_HELPER_H__

#include <gtk/gtk.h>
#include <sys/types.h>
#include <glade/glade.h>
#include <gconf/gconf-client.h>
#include <nautilus-burn-drive.h>

G_BEGIN_DECLS

#define OGMRIP_ERROR ogmrip_error_quark ()

typedef enum
{
  OGMRIP_ERROR_NONE,
  OGMRIP_ERROR_INTERNAL
} OGMRipError;

typedef struct
{
  gchar *name;
  gint fd;
} GFile;

/*
 * Glib
 */

gchar *  g_file_make_temp                (const gchar    *tmpl, 
                                          GError         **error);
void     g_file_unref                    (gchar          *filename,
                                          gboolean       do_unlink);
gboolean g_file_rename                   (const gchar    *old_name,
                                          const gchar    *new_name,
                                          GError         **error);
gboolean g_dir_remove                    (const gchar    *path,
                                          gboolean       recursive,
                                          GError         **error);

gboolean g_dir_make                      (const gchar    *path, 
                                          mode_t         mode, 
                                          GError         **error);
gchar *  g_dir_make_temp                 (const gchar    *tmpl, 
                                          mode_t         mode, 
                                          GError         **error);
gint64   g_fs_space_left                 (const gchar    *filename);
gchar *  g_fs_get_mount_point            (const gchar    *filename);

/*
 * GConf
 */

gint     gconf_client_get_int_default    (GConfClient    *client,
                                          const gchar    *key,
                                          gint           def);
gboolean gconf_client_get_bool_default   (GConfClient    *client,
                                          const gchar    *key,
                                          gboolean       def);
gchar *  gconf_client_get_string_default (GConfClient    *client,
                                          const gchar    *key,
                                          const gchar    *def);

/*
 * Gtk+
 */

void        gtk_window_set_parent           (GtkWindow      *window, 
                                             GtkWindow      *parent);
void        gtk_window_set_icon_from_stock  (GtkWindow      *window,
                                             const gchar    *stock_id);
gint        gtk_radio_button_get_active     (GtkRadioButton *radio);
void        gtk_radio_button_set_active     (GtkRadioButton *radio, 
                                             guint          index);
gboolean    gtk_tree_model_iter_prev        (GtkTreeModel   *tree_model,
                                             GtkTreeIter    *iter);
void        gtk_label_set_int               (GtkLabel       *label,
                                             gint           value);
gint        gtk_label_get_int               (GtkLabel       *label);
GtkWidget * gtk_box_get_child               (GtkBox         *box,
                                             guint          n);

/*
 * OGMDvd
 */

gboolean ogmdvd_is_valid_media            (const gchar             *device);
gboolean ogmdvd_tray_is_open              (const gchar             *device);
gboolean ogmdvd_tray_open                 (const gchar             *device);

/*
 * OGMRip
 */

GQuark      ogmrip_error_quark            (void);

gint        ogmrip_message_dialog         (GtkWindow           *parent, 
                                           GtkMessageType      type, 
                                           const gchar         *format, 
                                           ...);
void        ogmrip_statusbar_push         (GtkStatusbar        *statusbar,
                                           const gchar         *text);
void        ogmrip_statusbar_pop          (GtkStatusbar        *statusbar);

G_CONST_RETURN
gchar *     ogmrip_get_tmp_dir            (void);
void        ogmrip_set_tmp_dir            (const gchar         *dir);
gint        ogmrip_file_open_tmp          (const gchar         *tmpl,
                                           gchar               **name_used,
                                           GError              **error);

void        ogmrip_app_init               (GtkWindow           *window, 
                                           const gchar         *domain);
guint       ogmrip_app_add_ui_from_file   (GtkWindow           *window, 
                                           const gchar         *filename, 
                                           GError              **error);
void        ogmrip_app_add_actions        (GtkWindow           *window, 
                                           GtkActionEntry      *entries, 
                                           guint               n_entries);
void        ogmrip_app_set_menus          (GtkWindow           *window, 
                                           const gchar         *path);
void        ogmrip_app_set_statusbar      (GtkWindow           *window, 
                                           GtkWidget           *statusbar);
GtkWidget * ogmrip_app_get_widget         (GtkWindow           *window, 
                                           const gchar         *path);
GtkAction * ogmrip_app_action_connect     (GtkWindow           *window, 
                                           const gchar         *path, 
                                           GCallback           callback, 
                                           gpointer            user_data, 
                                           gboolean            sensitive, 
                                           gboolean            swapped);

G_END_DECLS

#endif /* __OGMRIP_HELPER_H__ */

